#include <gint/display.h>

#include "conf.h"
#include "level.h"
#include "camera.h"

#define VEC_PRECISE_HALF_DISP (Vec){DWIDTH * VEC_PRECISION / (2 * SCALE), DHEIGHT * VEC_PRECISION / (2 * SCALE)}

void level_step(const Level *level) {
}

void level_draw(const Level *level, Camera *camera) {
	layer_draw(level, camera, 0);
}

void layer_draw(const Level *level, Camera *camera, uint layer_id) {
	const uint8_t *layer = level->layers[layer_id];
	Vec display_tl, display_br;
	vec_cpy(&display_tl, camera->pos);
	vec_cpy(&display_br, display_tl);
	vec_sub(&display_tl, VEC_PRECISE_HALF_DISP);
	vec_add(&display_br, VEC_PRECISE_HALF_DISP);
	vec_div(&display_tl, TILE_SIZE);
	vec_div(&display_br, TILE_SIZE);
	int start_x = (display_tl.x > 0) ? display_tl.x : 0;
	int start_y = (display_tl.y > 0) ? display_tl.y : 0;
	int end_x = (display_br.x < level->width) ? display_br.x + 1 : level->width;
	int end_y = (display_br.y < level->height) ? display_br.y + 1 : level->height;
	for (int y = start_y; y < end_y; ++y) {
		for (int x = start_x; x < end_x; ++x) {
			const uint8_t cell = layer[x + y * level->width];
			#ifdef FX9860G
			const int color = C_LIGHT;
			#endif /* FX9860G */
			#ifdef FXCG50
			const int color = C_GREEN;
			#endif /* FXCG50 */
			if (cell == 1) {
				Vec tl = {x, y};
				Vec br;
				vec_mul(&tl, TILE_SIZE);
				vec_div(&tl, VEC_PRECISION);
				vec_sub(&tl, camera->offset);
				vec_mul(&tl, SCALE);
				vec_cpy(&br, tl);
				vec_add(&br, (Vec){TILE_SIZE / VEC_PRECISION * SCALE - 1,
					TILE_SIZE / VEC_PRECISION * SCALE - 1});
				vec_drect(tl, br, color);
			}
		}
	}
}
